Package: xlsx
Type: Package
Title: Read, Write, Format Excel 2007 and Excel 97/2000/XP/2003 Files
Version: 0.6.5
Imports: rJava, xlsxjars, grDevices, utils
LazyLoad: yes
Authors@R: 
    c(person(given = "Adrian",
             family = "Dragulescu",
             role = c("aut"),
             email = "adrian.dragulescu@gmail.com"),
      person(given = "Cole",
             family = "Arendt",
             role = c("aut", "cre"),
             email = "cole.arendt@outlook.com")
             )
Description: Provide R functions to read/write/format Excel 2007 and Excel 97/2000/XP/2003 file formats.
License: GPL-3
URL: https://github.com/colearendt/xlsx
BugReports: https://github.com/colearendt/xlsx/issues
SystemRequirements: java (>= 1.6)
Suggests: rprojroot, testthat, covr, tibble, knitr, rmarkdown
RoxygenNote: 7.1.0
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2020-11-10 12:06:13 UTC; carendt
Author: Adrian Dragulescu [aut],
  Cole Arendt [aut, cre]
Maintainer: Cole Arendt <cole.arendt@outlook.com>
Repository: CRAN
Date/Publication: 2020-11-10 15:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:20:54 UTC; unix
