# Generated by roxygen2: do not edit by hand

S3method("+",CellStyle)
S3method(CellBlock,default)
S3method(CellStyle,default)
export(Alignment)
export(BORDER_STYLES_)
export(Border)
export(CB.setBorder)
export(CB.setColData)
export(CB.setFill)
export(CB.setFont)
export(CB.setMatrixData)
export(CB.setRowData)
export(CELL_STYLES_)
export(CellBlock)
export(CellProtection)
export(CellStyle)
export(DataFormat)
export(FILL_STYLES_)
export(Fill)
export(Font)
export(HALIGN_STYLES_)
export(INDEXED_COLORS_)
export(VALIGN_STYLES_)
export(addAutoFilter)
export(addDataFrame)
export(addHyperlink)
export(addMergedRegion)
export(addPicture)
export(autoSizeColumn)
export(createCell)
export(createCellComment)
export(createFreezePane)
export(createRange)
export(createRow)
export(createSheet)
export(createSplitPane)
export(createWorkbook)
export(forceFormulaRefresh)
export(forcePivotTableRefresh)
export(getCellComment)
export(getCellStyle)
export(getCellValue)
export(getCells)
export(getRanges)
export(getRows)
export(getSheets)
export(get_java_tmp_dir)
export(is.Alignment)
export(is.Border)
export(is.CellBlock)
export(is.CellProtection)
export(is.CellStyle)
export(is.DataFormat)
export(is.Fill)
export(is.Font)
export(loadWorkbook)
export(printSetup)
export(read.xlsx)
export(read.xlsx2)
export(readColumns)
export(readRange)
export(readRows)
export(removeCellComment)
export(removeMergedRegion)
export(removeRow)
export(removeSheet)
export(saveWorkbook)
export(setCellStyle)
export(setCellValue)
export(setColumnWidth)
export(setPrintArea)
export(setRowHeight)
export(setZoom)
export(set_java_tmp_dir)
export(write.xlsx)
export(write.xlsx2)
import(rJava)
import(xlsxjars)
importFrom(grDevices,col2rgb)
importFrom(utils,unzip)
importFrom(utils,zip)
